// This parser reads tcpdump -w info from the stdin and
// sends it to a SDC node.

//#include "Parser.h"
#include <string.h>
#include <iostream.h>     
#include <stdio.h>

#include "Sender.h"

class FastParser {

    public:

    FastParser(Sender s): sender(s) {
      reserved=10;// the first 2 bytes are reserved for len, 8 bytes are reserved for a timestamp
      ch=0;
      sendBuffer=new char[65536+reserved]; // 2^16+10 proto overhead
    }

    void parse() {
      skip(24); // ignore the start header of tcpdump
      while(cin.peek()!=EOF) {
	  parsePacket();
      }
    }

    private:
    char *sendBuffer;
    unsigned int ipLen; // The number of ip bytes provided by tcpdump.
  unsigned int reserved; // 10 bytes of the sendbuffer are reserved for my protocol stuff.

    Sender sender;
    char ch;

    void parsePacket() {
      // parsing the tcpdump info
      putTimeIntIntoSendBuffer(2); // extract the first half of the timestamp.
      putTimeIntIntoSendBuffer(6); // extract the first half of the timestamp.
      putLenIntoSendBuffer(); // The number of ip bytes is extracted.
      skip(18); // Ignore additional info;
      putIPIntoSendBuffer();
      sendPacket(); // Send packet wise to the sdc node application.
    }

  void skip(int i) {
    // skip i characters from the stdin
    for(int j=0;j<i;j++) {
      cin.get(ch);
    }
  }

  void putTimeIntIntoSendBuffer(int pos) {
    // Change big-endian to litte endian.
    int up=pos+3;
    for(int i=up; i>=pos; i--) {
      cin.get(ch);
      sendBuffer[i]=ch;
    }
  }

  void putLenIntoSendBuffer() {
    // tcpdump also encodes the length of the dump packet.
    // Extract this (to get the ipLen and to find packet end).
    ipLen=readInt(4); // Can use more than 2 bytes. 3 would be enough, but 4 bytes must be consumed.
    ipLen-=14; // 14 bytes (probably ether stuff) are not provided.
    if(ipLen>65535) cerr << "\nIllegal ip len" << ipLen;
    // This is now the length of the IP packet, which fits in two bytes
    sendBuffer[0]=(char) (ipLen/256);
    sendBuffer[1]=(char) (ipLen%256);
  }

  void putIPIntoSendBuffer() {
    ch=cin.peek();
    if((ch>>4)!=4) cerr << "\nReading seems out of sync! IP version is not 4.";
     int read=0;
     while((cin.peek()!=EOF)&& (read<ipLen)) {
       cin.read((sendBuffer+reserved), ipLen);
       read+=cin.gcount();
     }
  }


  unsigned int readInt(int l) {
    // read l bytes (big endian) into an int.
    unsigned int ret=0;
    int h;
    short shift=0;
    for(int j=0;j<l;j++,shift+=8) {
      cin.get(ch);
      h=ch;
      if(h<0) h+=256;
      h=h<<shift;
      ret+=h;
    }
    return ret;
  }


    void sendPacket() {
	//cerr<< "\nSending " << ipLen+reserved << " bytes";
	sender.forward(sendBuffer,ipLen+reserved);
    }

};
